@@ -60,8 +60,6 @@ type Image interface {
 	Unmount() error
 	Format() error
 	Partition() error
-	Map() error
-	Unmap() error
 	BaseMount() string
 }
 
@@ -200,7 +198,7 @@ type BaseImage struct {
 
 // Mount mounts the image. This also maps the loop device.
 func (img *BaseImage) Mount() error {
-	if err := img.Map(); err != nil {
+	if err := img.doMap(); err != nil {
 		return err
 	}
 
@@ -260,6 +258,12 @@ func (img *BaseImage) Mount() error {
 
 // Unmount unmounts the image. This also unmaps the loop device.
 func (img *BaseImage) Unmount() error {
+	defer func() {
+		if isMapped(img.parts) {
+			fmt.Println("WARNING: could not unmap partitions")
+		}
+	}()
+
 	if img.baseMount == "" {
 		panic("No base mountpoint set")
 	}
@@ -285,11 +289,11 @@ func (img *BaseImage) Unmount() error {
 	}
 	img.baseMount = ""
 
-	return img.Unmap()
+	return img.doUnmap()
 }
 
-// Map maps the image to loop devices
-func (img *BaseImage) Map() error {
+// doMap maps the image to loop devices
+func (img *BaseImage) doMap() error {
 	if isMapped(img.parts) {
 		panic("cannot double map partitions")
 	}
@@ -332,20 +336,22 @@ func (img *BaseImage) Map() error {
 	return nil
 }
 
-//Unmap destroys loop devices for the partitions
-func (img *BaseImage) Unmap() error {
+// doUnmap destroys loop devices for the partitions
+func (img *BaseImage) doUnmap() error {
 	if img.baseMount != "" {
 		panic("cannot unmap mounted partitions")
 	}
 
 	for _, part := range img.parts {
-		if err := exec.Command("dmsetup", "clear", part.loop).Run(); err != nil {
-			return err
+		dmsetupCmd := []string{"dmsetup", "clear", part.loop}
+		if out, err := exec.Command(dmsetupCmd[0], dmsetupCmd[1:]...).CombinedOutput(); err != nil {
+			return &ErrExec{command: dmsetupCmd, output: out}
 		}
 	}
 
-	if err := exec.Command("kpartx", "-ds", img.location).Run(); err != nil {
-		return err
+	kpartxCmd := []string{"kpartx", "-ds", img.location}
+	if out, err := exec.Command(kpartxCmd[0], kpartxCmd[1:]...).CombinedOutput(); err != nil {
+		return &ErrExec{command: kpartxCmd, output: out}
 	}
 
 	unmapPartitions(img.parts)
@@ -355,12 +361,25 @@ func (img *BaseImage) Unmap() error {
 
 // Format formats the image following the partition types and labels them
 // accordingly.
-func (img BaseImage) Format() error {
+func (img BaseImage) Format() (err error) {
+	if err := img.doMap(); err != nil {
+		return err
+	}
+	defer func() {
+		if errUnmap := img.doUnmap(); errUnmap != nil {
+			if err == nil {
+				err = errUnmap
+			} else {
+				fmt.Println("WARNING: could not unmap partitions after error:", errUnmap)
+			}
+		}
+	}()
+
 	for _, part := range img.parts {
 		dev := filepath.Join("/dev/mapper", part.loop)
 
 		if part.fs == fsFat32 {
-			cmd := []string{"-F", "32", "-n", string(part.label)}
+			cmd := []string{"mkfs.vfat", "-F", "32", "-n", string(part.label)}
 
 			size, err := sectorSize(dev)
 			if err != nil {
@@ -373,12 +392,13 @@ func (img BaseImage) Format() error {
 
 			cmd = append(cmd, "-S", size, dev)
 
-			if out, err := exec.Command("mkfs.vfat", cmd...).CombinedOutput(); err != nil {
-				return fmt.Errorf("unable to create filesystem: %s", out)
+			if out, err := exec.Command(cmd[0], cmd[1:]...).CombinedOutput(); err != nil {
+				return &ErrExec{command: cmd, output: out}
 			}
 		} else {
-			if out, err := exec.Command("mkfs.ext4", "-F", "-L", string(part.label), dev).CombinedOutput(); err != nil {
-				return fmt.Errorf("unable to create filesystem: %s", out)
+			cmd := []string{"mkfs.ext4", "-F", "-L", string(part.label), dev}
+			if out, err := exec.Command(cmd[0], cmd[1:]...).CombinedOutput(); err != nil {
+				return &ErrExec{command: cmd, output: out}
 			}
 		}
 	}
